import random

import httpx
import json
import base64
import time
from typing_extensions import Unpack, Tuple
from http.cookies import SimpleCookie
from loguru import logger

from zzupy.typing import DeviceParams
from zzupy.utils import get_sign, _kget
from zzupy.supwisdom import Supwisdom
from zzupy.ecard import eCard
from zzupy.network import Network
from zzupy.exception import LoginException


class ZZUPy:
    def __init__(
        self, usercode: str, password: str, cookie: SimpleCookie | None = None
    ):
        """
        初始化一个 ZZUPy 对象

        :param str usercode: 学号
        :param str password: 密码
        :param SimpleCookie cookie: 统一认证 Cookie。目前必须包含 'userToken'，否则会抛出 ValueError
        :raises ValueError: Cookie 中缺少 'userToken' 时抛出
        """
        self._userToken = None
        # 固定值
        self._dynamicSecret = "supwisdom_eams_app_secret"
        self._dynamicToken = None
        self._refreshToken = None
        self._name = None
        self._isLogged = False
        self._DeviceParams = {
            "deviceName": "",
            "deviceId": "",
            "deviceInfo": "",
            "deviceInfos": "",
            "userAgentPrecursor": "",
        }
        self._usercode = usercode
        self._password = password
        logger.debug(f"已配置账户 {usercode}")
        # 初始化 HTTPX
        self._client = httpx.Client(follow_redirects=True)
        if cookie is SimpleCookie:
            for key, morsel in cookie.items():
                self._client.cookies.set(
                    key, morsel.value, morsel["domain"], morsel["path"]
                )
                if key == "userToken":
                    self._userToken = morsel.value
            if self._userToken is None:
                raise ValueError("Cookie 中缺少 'userToken'")
        logger.debug("已配置 HTTPX 实例")
        # 初始化类
        self.Network = Network(self)
        self.eCard = eCard(self)
        self.Supwisdom = Supwisdom(self)
        logger.debug("已配置类")
        logger.info(f"账户 {usercode} 初始化完成")

    def is_logged_in(self) -> bool:
        """
        判断是否已登录
        :return: 是否已登录
        :rtype: bool
        """
        if self._isLogged:
            return True
        else:
            return False

    def set_device_params(self, **kwargs: Unpack[DeviceParams]):
        """
        设置设备参数。这些参数都需要抓包获取，但其实可有可无，因为目前并没有观察到相关风控机制

        :param str deviceName: 设备名 ，位于 "passwordLogin" 请求的 User-Agent 中，组成为 '{appVersion}({deviceName})'
        :param str deviceId: 设备 ID ，
        :param str deviceInfo: 设备信息，位于名为 "X-Device-Info" 的请求头中
        :param str deviceInfos: 设备信息，位于名为 "X-Device-Infos" 的请求头中
        :param str userAgentPrecursor: 设备 UA 前体 ，只需要包含 "SuperApp" 或 "uni-app Html5Plus/1.0 (Immersed/38.666668)" 前面的部分
        """
        self._DeviceParams["deviceName"] = _kget(kwargs, "deviceName", "")
        self._DeviceParams["deviceId"] = _kget(kwargs, "deviceId", "")
        self._DeviceParams["deviceInfo"] = _kget(kwargs, "deviceInfo", "")
        self._DeviceParams["deviceInfos"] = _kget(kwargs, "deviceInfos", "")
        self._DeviceParams["userAgentPrecursor"] = _kget(kwargs, "deviceInfos", "")
        if self._DeviceParams["userAgentPrecursor"].endswith(" "):
            self._DeviceParams["userAgentPrecursor"] = self._DeviceParams[
                "userAgentPrecursor"
            ]
        else:
            self._DeviceParams["userAgentPrecursor"] = (
                self._DeviceParams["userAgentPrecursor"] + " "
            )
        logger.info("已配置设备参数")

    def login(
        self,
        appVersion: str = "SWSuperApp/1.0.39",
        appId: str = "com.supwisdom.zzu",
        osType: str = "android",
    ) -> Tuple[str, str]:
        """
        登录

        :param str appVersion: APP 版本 ，一般类似 "SWSuperApp/1.0.38" ，可自行更新版本号。
        :param str appId: APP 包名，一般不需要修改
        :param str osType: 系统类型，一般不需要修改
        :returns: Tuple[str, str]

            - **usercode** (str) – 学号
            - **name** (str) – 姓名
        :rtype: Tuple[str,str]
        :raises LoginException: 登录失败时抛出
        """
        logger.info(f"尝试登录账户 {self._usercode}")
        if self._client.cookies.get("userToken") is None:
            headers = {
                "User-Agent": f"{appVersion}({self._DeviceParams['deviceName']})",
                "Connection": "Keep-Alive",
                "Accept-Encoding": "gzip",
            }
            response = self._client.post(
                f"https://token.s.zzu.edu.cn/password/passwordLogin?username={self._usercode}&password={self._password}&appId={appId}&geo&deviceId={self._DeviceParams['deviceId']}&osType={osType}&clientId&mfaState",
                headers=headers,
            )
            logger.debug(f"/passwordLogin 请求响应体: {response.text}")
            # 获取 userToken 和 refreshToken
            try:
                self._userToken = json.loads(response.text)["data"]["idToken"]
                # 我也不知道 refreshToken 有什么用，但先存着吧
                self._client.cookies.set(
                    "userToken", self._userToken, ".zzu.edu.cn", "/"
                )
                self._refreshToken = json.loads(response.text)["data"]["refreshToken"]
            except Exception as exc:
                logger.error(
                    "从 /passwordLogin 请求中提取 userToken 和 refreshToken 失败"
                )
                raise LoginException("登录失败, 通过 DEBUG 日志获得更多信息") from exc
        else:
            logger.info(f"userToken 已设置，跳过帐密登录")

        headers = {
            "User-Agent": self._DeviceParams["userAgentPrecursor"] + "SuperApp",
            "Accept": "application/json, text/plain, */*",
            "Accept-Encoding": "gzip, deflate, br, zstd",
            "Content-Type": "application/x-www-form-urlencoded",
            "sec-ch-ua": '"Not/A)Brand";v="8", "Chromium";v="126", "Android WebView";v="126"',
            "sec-ch-ua-mobile": "?1",
            "sec-ch-ua-platform": '"Android"',
            "Origin": "https://jw.v.zzu.edu.cn",
            "X-Requested-With": "com.supwisdom.zzu",
            "Sec-Fetch-Site": "same-origin",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Dest": "empty",
            "Referer": "https://jw.v.zzu.edu.cn/app-web/",
            "Accept-Language": "zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7",
        }
        data = {
            "random": int(random.uniform(10000, 99999)),
            "timestamp": int(round(time.time() * 1000)),
            "userToken": self._userToken,
        }
        # 计算 sign 并将其加入 data
        params = ""
        for key in data.keys():
            params += f"{key}={data[key]}&"
        params = params[:-1]
        sign = get_sign(self._dynamicSecret, params)
        data["sign"] = sign

        response = self._client.post(
            "https://jw.v.zzu.edu.cn/app-ws/ws/app-service/super/app/login-token",
            headers=headers,
            data=data,
        )
        logger.debug(f"/login-token 请求响应体: {response.text}")
        try:
            self._dynamicSecret = json.loads(
                base64.b64decode(json.loads(response.text)["business_data"])
            )["secret"]
            self._dynamicToken = json.loads(
                base64.b64decode(json.loads(response.text)["business_data"])
            )["token"]
            self._name = json.loads(
                base64.b64decode(json.loads(response.text)["business_data"])
            )["user_info"]["user_name"]
        except Exception as exc:
            logger.error(
                "从 /login-token 请求中提取 dynamicSecret 、 dynamicToken 和用户信息失败"
            )
            raise LoginException("登录失败, 通过 DEBUG 日志获得更多信息") from exc
        self.eCard._start_token_refresh_timer()
        self._isLogged = True
        logger.info(f"账户 {self._usercode} 登录成功")
        return self._usercode, self._name



import base64
import datetime
import json
import random
import time
import httpx

from zzupy.utils import get_sign


class Supwisdom:
    def __init__(self, parent):
        self._parent = parent

    def get_courses(self, start_date: str) -> str:
        """
        获取课程表

        :param str start_date: 课表的开始日期，格式必须为 %Y-%m-%d ，且必须为某一周周一，否则课表会时间错乱
        :return: 返回 Json 格式的课程表数据
        :rtype: str
        """

        headers = {
            "User-Agent": self._parent._DeviceParams["userAgentPrecursor"] + "SuperApp",
            "Accept": "application/json, text/plain, */*",
            "Accept-Encoding": "gzip, deflate, br, zstd",
            "Content-Type": "application/x-www-form-urlencoded",
            "sec-ch-ua": '"Not/A)Brand";v="8", "Chromium";v="126", "Android WebView";v="126"',
            "sec-ch-ua-mobile": "?1",
            "token": self._parent._dynamicToken,
            "sec-ch-ua-platform": '"Android"',
            "Origin": "https://jw.v.zzu.edu.cn",
            "X-Requested-With": "com.supwisdom.zzu",
            "Sec-Fetch-Site": "same-origin",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Dest": "empty",
            "Referer": "https://jw.v.zzu.edu.cn/app-web/",
            "Accept-Language": "zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7",
        }

        data = {
            "biz_type_id": "1",
            "end_date": (
                datetime.datetime.strptime(start_date, "%Y-%m-%d")
                + datetime.timedelta(days=6)
            ).strftime("%Y-%m-%d"),
            "random": int(random.uniform(10000, 99999)),
            "semester_id": "152",
            "start_date": start_date,
            "timestamp": int(round(time.time() * 1000)),
            "token": self._parent._userToken,
        }

        params = ""
        for key in data.keys():
            params += f"{key}={data[key]}&"
        params = params[:-1]
        sign = get_sign(self._parent._dynamicSecret, params)
        data["sign"] = sign

        response = self._parent._client.post(
            "https://jw.v.zzu.edu.cn/app-ws/ws/app-service/student/course/schedule/get-course-tables",
            headers=headers,
            data=data,
        )
        coursesJson = (
            base64.b64decode(json.loads(response.text)["business_data"])
        ).decode("utf-8")
        sorted_courses_json = sorted(
            json.loads(coursesJson),
            key=lambda x: (
                x["date"],
                datetime.datetime.strptime(x["start_time"], "%H:%M"),
            ),
        )
        return json.dumps(sorted_courses_json).encode("utf-8").decode("unicode_escape")



from os import PathLike
from loguru import logger
import sys

logger.remove()


def configure_logging(
    enable: bool = False,
    level: str = "INFO",
    log_to_console: bool = True,
    log_file: str | PathLike[str] = None,
    rotation: str = "10 MB",
    retention: str = "1 week",
    log_format: str = "<green>{time:YYYY-MM-DD HH:mm:ss}</green> - <lvl>{level:^8}</> - <cyan>{name:^12}</cyan> : <cyan>{module:^7}</cyan> : <cyan>{line:^4}</cyan> - <lvl>{message}</>",
) -> logger:
    """

    :param enable: 是否启用日志记录
    :param level: 日志级别 ("TRACE", "DEBUG", "INFO", "SUCCESS", "WARNING", "ERROR", "CRITICAL")
    :param log_to_console: 是否输出到控制台
    :param log_file: 日志文件路径，如果为 None 则不记录到文件
    :param rotation: 日志轮转条件，例如 "10 MB", "1 day", "1 week"
    :param retention: 保留历史日志的时间
    :param log_format: 日志格式
    :return: logger 对象
    """
    # 确保移除所有现有的处理器
    logger.remove()

    if not enable:
        # 如果不启用日志，仅返回logger
        return logger

    # 添加控制台处理器
    if log_to_console:
        logger.add(sys.stderr, level=level, format=log_format)

    # 添加文件处理器
    if log_file:
        logger.add(
            log_file,
            rotation=rotation,  # 轮转条件
            retention=retention,  # 保留策略
            level=level,
            format=log_format,
        )

    return logger



class ZZUPyException(Exception):
    """ZZUPy项目的基类异常"""

    pass


class LoginException(ZZUPyException):
    """登录相关异常"""

    pass


class DefaultRoomException(ZZUPyException):
    """默认房间相关异常"""

    pass


class ECardTokenException(ZZUPyException):
    """校园卡 Token 相关异常"""

    pass


class PermissionException(ZZUPyException):
    """校园卡 Token 相关异常"""

    pass



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html
import sys
import os
from os.path import abspath, dirname

sys.path.insert(0, dirname(dirname(dirname(abspath(__file__)))))
sys.path.insert(0, os.path.abspath("../../zzupy"))
# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

project = "ZZU.Py"
copyright = "2024-2025, Illustar0"
author = "Illustar0"
release = "3.0.0"
autoclass_content = "both"


# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.intersphinx",
    "sphinx.ext.coverage",
    "myst_parser",
    "sphinx.ext.githubpages",
    "sphinx.ext.viewcode",
    "sphinx.ext.mathjax",
]

templates_path = ["_templates"]
exclude_patterns = ["log.py", "log.rst"]

language = "zh_cn"

# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = "sphinx_book_theme"
html_static_path = ["_static"]



import base64
import random
import time
import httpx
import re
import json
from fake_useragent import UserAgent
from bs4 import BeautifulSoup
from typing_extensions import Tuple

from zzupy.utils import get_ip_by_interface, get_default_interface


class Network:
    def __init__(self, parent):
        self._parent = parent
        self.account = self._parent._usercode
        self._JSessionID = ""
        self._checkcode = ""
        self.system_ua = ""
        self.system_loginurl = ""

    def portal_auth(
        self,
        interface: str = get_default_interface(),
        authurl="http://10.2.7.8:801",
        ua=UserAgent().random,
        isp="campus",
    ) -> Tuple[str, bool, str]:
        """
        进行校园网认证

        :param str interface: 网络接口名
        :param str authurl: PortalAuth 服务器。根据情况修改
        :param str ua: User-Agent
        :param str isp: 运营商。可选项：campus,cm
        :returns: Tuple[str, bool, str]

            - **interface** (str) – 本次认证调用的网络接口。
            - **success** (bool) – 认证是否成功。(不可信，有时失败仍可正常上网)
            - **msg** (str) – 服务端返回信息。
        :rtype: Tuple[str,bool,str]
        """
        if isp == "campus":
            self.account = self._parent._usercode
        elif isp == "ct":
            self.account = self._parent._usercode + "@cmcc"
        elif isp == "cu":
            self.account = self._parent._usercode + "@cmcc"
        elif isp == "cm":
            self.account = self._parent._usercode + "@cmcc"
        else:
            self.account = self._parent._usercode
        transport = httpx.HTTPTransport(local_address=get_ip_by_interface(interface))
        local_client = httpx.Client(transport=transport)
        self._chkstatus(local_client, authurl, ua)
        self._loadConfig(local_client, interface, authurl, ua)
        return self._auth(local_client, interface, authurl, ua)

    def _auth(
        self,
        client,
        interface,
        baseURL,
        ua,
    ):
        headers = {
            "Accept": "*/*",
            "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
            "Connection": "keep-alive",
            "Referer": "http://10.2.7.8/",
            "User-Agent": ua,
        }
        params = [
            ("callback", "dr1003"),
            ("login_method", "1"),
            ("user_account", f",0,{self.account}"),
            (
                "user_password",
                base64.b64encode(self._parent._password.encode()).decode(),
            ),
            ("wlan_user_ip", get_ip_by_interface(interface)),
            ("wlan_user_ipv6", ""),
            ("wlan_user_mac", "000000000000"),
            ("wlan_ac_ip", ""),
            ("wlan_ac_name", ""),
            ("jsVersion", "4.2.1"),
            ("terminal_type", "1"),
            ("lang", "zh-cn"),
            ("v", str(random.randint(500, 10499))),
            ("lang", "zh"),
        ]
        response = client.get(
            f"{baseURL}/eportal/portal/login", params=params, headers=headers
        )
        res_json = json.loads(re.findall(r"dr1003\((.*?)\);", response.text)[0])
        if res_json["result"] == 0:
            success = False
        else:
            success = True
        return interface, success, res_json["msg"]

    # 现在发现可有可无好像
    def _chkstatus(self, client, baseURL, ua):
        headers = {
            "Accept": "*/*",
            "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
            "Connection": "keep-alive",
            "Referer": "http://10.2.7.8/a79.htm",
            "User-Agent": ua,
        }

        params = {
            "callback": "dr1002",
            "jsVersion": "4.X",
            "v": str(random.randint(500, 10499)),
            "lang": "zh",
        }
        client.get(
            re.sub(r":\d+", "", baseURL) + "/drcom/chkstatus",
            params=params,
            headers=headers,
        )

    def _loadConfig(self, client, interface, baseURL, ua):
        headers = {
            "Accept": "*/*",
            "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
            "Connection": "keep-alive",
            "Referer": "http://10.2.7.8/",
            "User-Agent": ua,
        }

        params = {
            "callback": "dr1001",
            "program_index": "",
            "wlan_vlan_id": "1",
            "wlan_user_ip": base64.b64encode(
                get_ip_by_interface(interface).encode()
            ).decode(),
            "wlan_user_ipv6": "",
            "wlan_user_ssid": "",
            "wlan_user_areaid": "",
            "wlan_ac_ip": "",
            "wlan_ap_mac": "000000000000",
            "gw_id": "000000000000",
            "jsVersion": "4.X",
            "v": str(random.randint(500, 10499)),
            "lang": "zh",
        }
        client.get(
            f"{baseURL}/eportal/portal/page/loadConfig", params=params, headers=headers
        )

    def login(
        self, loginurl: str = "http://10.2.7.16:8080", ua: str = UserAgent().random
    ):
        """
        登录自助服务平台

        :param str loginurl: 自助服务平台的登录 URL
        :param str ua: User Agent
        """
        self.system_ua = ua
        self.system_loginurl = loginurl
        headers = {
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
            "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
            "Cache-Control": "no-cache",
            "Pragma": "no-cache",
            "Proxy-Connection": "keep-alive",
            "Upgrade-Insecure-Requests": "1",
            "User-Agent": self.system_ua,
        }

        response = httpx.get(
            f"{self.system_loginurl}/Self/login/",
            headers=headers,
            verify=False,
            follow_redirects=False,
        )
        self._JSessionID = response.headers["set-cookie"].split("=")[1].split(";")[0]
        soup = BeautifulSoup(response.text, features="html.parser")
        self._checkcode = soup.find_all("input", attrs={"name": "checkcode"})[0][
            "value"
        ]
        cookies = {
            "JSESSIONID": self._JSessionID,
        }

        headers = {
            "Accept": "image/avif,image/webp,image/apng,image/svg+xml,image/*,*/*;q=0.8",
            "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
            "Cache-Control": "no-cache",
            "Pragma": "no-cache",
            "Proxy-Connection": "keep-alive",
            "Referer": f"{self.system_loginurl}/Self/login/",
            "User-Agent": self.system_ua,
        }

        params = {
            "t": str(random.random()),
        }

        httpx.get(
            f"{self.system_loginurl}/Self/login/randomCode",
            params=params,
            cookies=cookies,
            headers=headers,
            verify=False,
        )

        headers = {
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
            "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
            "Cache-Control": "no-cache",
            "Content-Type": "application/x-www-form-urlencoded",
            "Origin": f"{self.system_loginurl}",
            "Pragma": "no-cache",
            "Proxy-Connection": "keep-alive",
            "Referer": f"{self.system_loginurl}/Self/login/",
            "Upgrade-Insecure-Requests": "1",
            "User-Agent": self.system_ua,
        }

        data = {
            "foo": "",
            "bar": "",
            # 我草，太坏了，这个 checkcode 居然是直接动态写死在网页里的，搞得我扣了半天算法抠不出来
            "checkcode": self._checkcode,
            "account": self._parent._usercode,
            "password": self._parent._password,
            "code": "",
        }
        httpx.post(
            f"{self.system_loginurl}/Self/login/verify;jsessionid={self._JSessionID}",
            cookies=cookies,
            headers=headers,
            data=data,
            verify=False,
        )

    def get_online_devices(self) -> str:
        """
        获取全部在线设备

        :return: Json 格式的在线设备数据
        :rtype: str
        """
        cookies = {
            "JSESSIONID": self._JSessionID,
        }
        headers = {
            "Accept": "application/json, text/javascript, */*; q=0.01",
            "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
            "Cache-Control": "no-cache",
            "Content-Type": "application/json",
            "Pragma": "no-cache",
            "Proxy-Connection": "keep-alive",
            "Referer": f"{self.system_loginurl}/Self/dashboard",
            "User-Agent": self.system_ua,
            "X-Requested-With": "XMLHttpRequest",
        }

        params = {
            "t": str(random.random()),
            "order": "asc",
            "_": str(int(time.time())),
        }

        response = httpx.get(
            f"{self.system_loginurl}/Self/dashboard/getOnlineList",
            params=params,
            cookies=cookies,
            headers=headers,
            verify=False,
        )
        return response.text

    def get_total_traffic(self) -> int:
        """
        获取消耗的流量

        :return: 消耗的流量，单位为 MB
        :rtype: int
        """

        cookies = {
            "JSESSIONID": self._JSessionID,
        }

        headers = {
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
            "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
            "Cache-Control": "no-cache",
            "Pragma": "no-cache",
            "Proxy-Connection": "keep-alive",
            "Referer": f"{self.system_loginurl}/Self/login/",
            "Upgrade-Insecure-Requests": "1",
            "User-Agent": self.system_ua,
        }

        response = httpx.get(
            f"{self.system_loginurl}/Self/dashboard",
            cookies=cookies,
            headers=headers,
            verify=False,
        )
        soup = BeautifulSoup(response.text, features="html.parser")
        return int(soup.find_all("dt")[1].text.strip().split()[0])

    def get_used_time(self) -> int:
        """
        获取使用时间

        :return int: 使用时间，单位为 分钟
        :rtype: int
        """
        cookies = {
            "JSESSIONID": self._JSessionID,
        }

        headers = {
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
            "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
            "Cache-Control": "no-cache",
            "Pragma": "no-cache",
            "Proxy-Connection": "keep-alive",
            "Referer": f"{self.system_loginurl}/Self/login/",
            "Upgrade-Insecure-Requests": "1",
            "User-Agent": self.system_ua,
        }

        response = httpx.get(
            f"{self.system_loginurl}/Self/dashboard",
            cookies=cookies,
            headers=headers,
            verify=False,
        )
        soup = BeautifulSoup(response.text, features="html.parser")
        return int(soup.find_all("dt")[0].text.strip().split()[0])

    def logout_device(self, sessionid: str) -> bool:
        """

        :param str sessionid: sessionid,可通过 get_online_devices() 获取
        :return: 成功或失败
        :rtype: bool
        """
        cookies = {
            "JSESSIONID": self._JSessionID,
        }

        headers = {
            "Accept": "*/*",
            "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
            "Cache-Control": "no-cache",
            "Pragma": "no-cache",
            "Proxy-Connection": "keep-alive",
            "Referer": f"{self.system_loginurl}/Self/dashboard",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36",
            "X-Requested-With": "XMLHttpRequest",
        }

        params = {
            "t": str(random.random()),
            "sessionid": sessionid,
        }

        response = httpx.get(
            f"{self.system_loginurl}/Self/dashboard/tooffline",
            params=params,
            cookies=cookies,
            headers=headers,
            verify=False,
        )
        if response.status_code == 200 and json.loads(response.text)["success"]:
            return True
        else:
            # Log
            return False



import base64
import json
import threading
import time
import warnings
from urllib.parse import urlparse, parse_qs

import gmalg
from typing_extensions import Tuple
from loguru import logger

from zzupy.exception import DefaultRoomException, ECardTokenException
from zzupy.utils import sm4_decrypt_ecb, check_permission


class eCard:
    def __init__(self, parent):
        self._parent = parent
        self._eCardAccessToken = None
        self._eCardRefreshToken = None
        self._JSessionID = None
        self._tid = None
        self._orgId = None

    def _start_token_refresh_timer(self):
        self._JSessionID, self._tid, self._orgId = self._get_jsession_id()
        self._eCardAccessToken, self._eCardRefreshToken = self._get_ecard_access_token()
        # 每 45 分钟（2700 秒）执行一次
        self._timer = threading.Timer(2700, self._start_token_refresh_timer)
        self._timer.daemon = True
        self._timer.start()

    def _get_jsession_id(self):
        headers = {
            "User-Agent": self._parent._DeviceParams["userAgentPrecursor"] + "SuperApp",
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7",
            "Accept-Encoding": "gzip, deflate, br, zstd",
            "sec-ch-ua": '"Android WebView";v="129", "Not=A?Brand";v="8", "Chromium";v="129"',
            "sec-ch-ua-mobile": "?1",
            "sec-ch-ua-platform": '"Android"',
            "Upgrade-Insecure-Requests": "1",
            "x-id-token": self._parent._userToken,
            "X-Requested-With": "com.supwisdom.zzu",
            "Sec-Fetch-Site": "none",
            "Sec-Fetch-Mode": "navigate",
            "Sec-Fetch-User": "?1",
            "Sec-Fetch-Dest": "document",
            "Accept-Language": "zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7",
        }

        params = {
            "host": "11",
            "org": "2",
            "token": self._parent._userToken,
        }
        logger.debug("尝试获取 JSessionID 和 tid")
        response = self._parent._client.get(
            "https://ecard.v.zzu.edu.cn/server/auth/host/open",
            params=params,
            headers=headers,
            follow_redirects=False,
        )
        logger.debug(f"/auth/host/open 请求响应头：{response.headers}")
        try:
            return (
                response.cookies.get("JSESSIONID"),
                parse_qs(urlparse(response.headers["location"]).query)["tid"][0],
                parse_qs(urlparse(response.headers["location"]).query)["orgId"][0],
            )
        except Exception as exc:
            logger.error("获取 JSessionID 和 tid 失败")
            raise ECardTokenException(
                "获取 JSessionID 和 tid 失败, 通过 DEBUG 日志获得更多信息"
            ) from exc

    def _get_ecard_access_token(self):
        headers = {
            "User-Agent": self._parent._DeviceParams["userAgentPrecursor"] + "SuperApp",
            "Accept-Encoding": "gzip, deflate, br, zstd",
            "Content-Type": "application/json",
            "sec-ch-ua-platform": '"Android"',
            "sec-ch-ua": '"Android WebView";v="129", "Not=A?Brand";v="8", "Chromium";v="129"',
            "sec-ch-ua-mobile": "?1",
            "Origin": "https://ecard.v.zzu.edu.cn",
            "X-Requested-With": "com.supwisdom.zzu",
            "Sec-Fetch-Site": "same-origin",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Dest": "empty",
            "Referer": f"https://ecard.v.zzu.edu.cn/?tid={self._tid}&{self._orgId}",
            "Accept-Language": "zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7",
        }

        data = {
            "tid": self._tid,
        }
        logger.debug(headers)
        response = self._parent._client.post(
            "https://ecard.v.zzu.edu.cn/server/auth/getToken",
            headers=headers,
            json=data,
        )
        logger.debug(f"/auth/getToken 请求响应体：{response.text}")
        try:
            return json.loads(response.text)["resultData"]["accessToken"], json.loads(
                response.text
            )["resultData"]["refreshToken"]
        except Exception as exc:
            logger.error("获取 eCardAccessToken 失败")
            raise ECardTokenException(
                "获取 eCardAccessToken 失败, 通过 DEBUG 日志获得更多信息"
            ) from exc

    def get_default_room(self) -> str:
        """
        获取账户默认房间

        :returns: 默认的房间
        """
        check_permission(self._parent)
        logger.debug("尝试获取默认 room")
        headers = {
            "User-Agent": self._parent._DeviceParams["userAgentPrecursor"] + "SuperApp",
            "Content-Type": "application/json",
            "sec-ch-ua-platform": '"Android"',
            "Authorization": self._eCardAccessToken,
            "sec-ch-ua": '"Not(A:Brand";v="99", "Android WebView";v="133", "Chromium";v="133"',
            "sec-ch-ua-mobile": "?1",
            "Origin": "https://ecard.v.zzu.edu.cn",
            "X-Requested-With": "com.supwisdom.zzu",
            "Sec-Fetch-Site": "same-origin",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Dest": "empty",
            "Referer": f"https://ecard.v.zzu.edu.cn/?tid={self._tid}&{self._orgId}",
            "Accept-Language": "zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7",
        }

        data = {
            "utilityType": "electric",
        }

        response = self._parent._client.post(
            "https://ecard.v.zzu.edu.cn/server/utilities/config",
            headers=headers,
            json=data,
        )
        try:
            room = json.loads(response.text)["resultData"]["location"]["room"]
            logger.debug(f"默认 room 为 {room}")
            return room
        except Exception as exc:
            logger.error("获取默认 room 失败")
            raise DefaultRoomException(
                "获取默认 room 失败, 通过 DEBUG 日志获得更多信息"
            ) from exc

    def recharge_energy(
        self, payment_password: str, amt: int, room: str | None = None
    ) -> Tuple[bool, str]:
        """
        为 room 充值电费

        :param str room: 房间 ID 。格式应为 'areaid-buildingid--unitid-roomid'，可通过 get_room_dict() 获取
        :param str payment_password: 支付密码
        :param int amt: 充值金额
        :returns: Tuple[bool, str]

            - **success** (bool) – 充值是否成功
            - **msg** (str) – 服务端返回信息。
        :rtype: Tuple[bool,str]
        """
        check_permission(self._parent)
        room = self.get_default_room() if room is None else room

        headers = {
            "Accept": "*/*",
            "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
            "Authorization": self._eCardAccessToken,
            "Cache-Control": "no-cache",
            "Connection": "keep-alive",
            "Content-Type": "application/json",
            "Origin": "https://ecard.v.zzu.edu.cn",
            "Pragma": "no-cache",
            "Referer": f"https://ecard.v.zzu.edu.cn/?tid={self._tid}&{self._orgId}",
            "Sec-Fetch-Dest": "empty",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Site": "same-origin",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36",
            "sec-ch-ua": '"Google Chrome";v="131", "Chromium";v="131", "Not_A Brand";v="24"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Windows"',
        }
        response = self._parent._client.post(
            "https://ecard.v.zzu.edu.cn/server/auth/getEncrypt",
            headers=headers,
        )
        pay_id = json.loads(response.text)["resultData"]["id"]
        encrypted_public_key = json.loads(response.text)["resultData"]["publicKey"]
        # 解密被加密的公钥
        public_key = sm4_decrypt_ecb(
            base64.b64decode(encrypted_public_key),
            bytes.fromhex("773638372d392b33435f48266a655f35"),
        )
        # 请求体明文
        json_data = {
            "utilityType": "electric",
            "payCode": "06",
            "password": payment_password,
            "amt": str(amt),
            "timestamp": int(round(time.time() * 1000)),
            "bigArea": "",
            "area": room.split("--")[0].split("-")[0],
            "building": room.split("--")[0].split("-")[1],
            "unit": "",
            "level": room.split("--")[1].split("-")[0],
            "room": room,
            "subArea": "",
            "customfield": {},
        }
        json_string = json.dumps(json_data, separators=(",", ":"))
        # 加密 params
        sm2 = gmalg.SM2(pk=bytes.fromhex(public_key))
        encrypted_params = sm2.encrypt(json_string.encode())
        data = {"id": pay_id, "params": (encrypted_params.hex())[2:]}
        response = self._parent._client.post(
            "https://ecard.v.zzu.edu.cn/server/utilities/pay",
            headers=headers,
            json=data,
        )
        return (
            json.loads(response.text)["success"],
            json.loads(response.text)["message"],
        )

    def get_balance(self) -> float:
        """
        获取校园卡余额

        :return: 校园卡余额
        :rtype: float
        """
        check_permission(self._parent)

        headers = {
            "User-Agent": self._parent._DeviceParams["userAgentPrecursor"]
            + "uni-app Html5Plus/1.0 (Immersed/38.666668)",
            "Connection": "Keep-Alive",
            "Accept": "application/json",
            "Accept-Encoding": "gzip",
            "X-Device-Info": self._parent._DeviceParams["deviceInfo"],
            "X-Device-Infos": self._parent._DeviceParams["deviceInfos"],
            "X-Id-Token": self._parent._userToken,
            "X-Terminal-Info": "app",
            "Content-Type": "application/x-www-form-urlencoded",
        }

        response = self._parent._client.get(
            "https://info.s.zzu.edu.cn/portal-api/v1/thrid-adapter/get-person-info-card-list",
            headers=headers,
        )
        return float(json.loads(response.text)["data"][1]["amount"])

    def get_room_dict(self, id: str) -> dict:
        """
        获取房间的字典

        :param str id: 已知房间 ID 。例如: '', '99', '99-12', '99-12--33'
        :return: 对应的字典
        :rtype: dict
        """

        check_permission(self._parent)
        num = id.count("-")
        if num == 0 and id == "":
            area = building = level = ""
            location_type = "bigArea"
        elif num == 0 and id != "":
            building = level = ""
            area = id
            location_type = "building"
        elif num == 1:
            area, building = id.split("-")
            level = ""
            location_type = "unit"
        elif num == 3:
            area, building = id.split("--")[0].split("-")
            level = id.split("--")[1]
            location_type = "room"
        else:
            raise ValueError("参数不合法")

        headers = {
            "User-Agent": self._parent._DeviceParams["userAgentPrecursor"] + "SuperApp",
            "Accept-Encoding": "gzip, deflate, br, zstd",
            "Content-Type": "application/json",
            "sec-ch-ua-platform": '"Android"',
            "Authorization": self._eCardAccessToken,
            "sec-ch-ua": '"Android WebView";v="129", "Not=A?Brand";v="8", "Chromium";v="129"',
            "sec-ch-ua-mobile": "?1",
            "Origin": "https://ecard.v.zzu.edu.cn",
            "X-Requested-With": "com.supwisdom.zzu",
            "Sec-Fetch-Site": "same-origin",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Dest": "empty",
            "Referer": f"https://ecard.v.zzu.edu.cn/?tid={self._tid}&{self._orgId}",
            "Accept-Language": "zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7",
        }

        data = {
            "utilityType": "electric",
            "locationType": location_type,
            "bigArea": "",
            "area": area,
            "building": building,
            "unit": "",
            "level": level,
            "room": "",
            "subArea": "",
        }

        response = self._parent._client.post(
            "https://ecard.v.zzu.edu.cn/server/utilities/location",
            headers=headers,
            json=data,
        )
        RoomDict = {}
        for i in range(len(json.loads(response.text)["resultData"]["locationList"])):
            RoomDict[
                json.loads(response.text)["resultData"]["locationList"][i]["id"]
            ] = json.loads(response.text)["resultData"]["locationList"][i]["name"]
        return RoomDict

    def get_remaining_energy(self, room: str | None = None) -> float:
        """
        获取剩余电量

        :param str room: 房间 ID 。格式应为 'areaid-buildingid--unitid-roomid'，可通过 get_room_dict() 获取
        :return: 剩余能源
        :rtype: float
        """

        check_permission(self._parent)
        room = self.get_default_room() if room is None else room

        headers = {
            "User-Agent": self._parent._DeviceParams["userAgentPrecursor"] + "SuperApp",
            "Accept-Encoding": "gzip, deflate, br, zstd",
            "Content-Type": "application/json",
            "sec-ch-ua-platform": '"Android"',
            "Authorization": self._eCardAccessToken,
            "sec-ch-ua": '"Android WebView";v="129", "Not=A?Brand";v="8", "Chromium";v="129"',
            "sec-ch-ua-mobile": "?1",
            "Origin": "https://ecard.v.zzu.edu.cn",
            "X-Requested-With": "com.supwisdom.zzu",
            "Sec-Fetch-Site": "same-origin",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Dest": "empty",
            "Referer": f"https://ecard.v.zzu.edu.cn/?tid={self._tid}&{self._orgId}",
            "Accept-Language": "zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7",
        }

        data = {
            "utilityType": "electric",
            "bigArea": "",
            "area": room.split("--")[0].split("-")[0],
            "building": room.split("--")[0].split("-")[1],
            "unit": "",
            "level": room.split("--")[1].split("-")[0],
            "room": room,
            "subArea": "",
        }

        response = self._parent._client.post(
            "https://ecard.v.zzu.edu.cn/server/utilities/account",
            headers=headers,
            json=data,
        )
        return float(
            json.loads(response.text)["resultData"]["templateList"][3]["value"]
        )

    def get_remaining_power(self, room: str | None = None) -> float:
        logger.warning("get_remaining_power() 已废弃，请使用 get_remaining_energy()")
        warnings.warn(
            "get_remaining_power() is deprecated, please use get_remaining_energy()",
            DeprecationWarning,
        )
        return self.get_remaining_energy(room)

    def recharge_electricity(
        self, payment_password: str, amt: int, room: str | None = None
    ) -> Tuple[bool, str]:
        logger.warning("recharge_electricity() 已废弃，请使用 recharge_energy()")
        warnings.warn(
            "recharge_electricity() is deprecated, please use recharge_energy()",
            DeprecationWarning,
        )
        return self.recharge_energy(payment_password, amt, room)
